/**********************************************************************-*-C-*-
 *
 * reveal.c - This file is part of `xradio'
 *
 * Copyright (C) 1996 Pat Thoyts (thoyts@bsa.bristol.ac.uk)
 *
 * These functions are all involved with deciding what data to send to which
 * io ports. The data is all the be found in `reveal.h'. Access to the ports
 * is all through the functions in `portlib.h'.
 *
 * The Reveal radio card has two possible IO bases. It is normally set to 
 * 0x30C but can be set to 0x20C using a jumper. It doesn't need an IRQ. The
 * range required is from 0x30C to 0x310 (or 0x20C to 0x210).
 *
 * Almost all writes to the card when using the DOS program `fmradio.exe'
 * appear to be through 0x30F:0x310
 *
 * If you need to change the default port and are looking here - read the
 * README file! and go look at the reveal.h file.
 *
 *****************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include "reveal.h"
#include "portlib.h"

/* Functions private to this file. */
static int instruct_radio ( RadioCommand cmd[] );
static int instruct_radio_fast ( RadioCommand cmd[] );
static int tune_radio( float freq );
static int legal_freq( float freq );

/*
 * The `poweron' sequence seems to reset the volume, though not the current 
 * frequency. The `radio_on' sequence switches the radio on but without
 * resetting the volume - hence I tend to use `radio_on' for unmuting the
 * radio.
 */
int
radio_poweron(float freq)
{
  RadioCommand cmd[]  = RADIO_PWR;
  if (!(legal_freq(freq))) {
    freq = RADIO_DEFHz;
  }
  tune_radio(freq);
  return instruct_radio(cmd);
}

int
radio_on( void )
{
  RadioCommand cmd[]  = RADIO_ON;
  return instruct_radio(cmd);
}

/*
 * The `off' sequence mutes the radio without resetting anything. This is used
 * for both mute and to switch the device off.
 */
int
radio_off( void )
{
  RadioCommand cmd[] = RADIO_OFF;
  return instruct_radio(cmd);
}

/*
 * Increases the volume output.
 */
int radio_loud(int num)
{
  RadioCommand cmd[] = RADIO_INC;
  int cn = 0, r = 0;

  do {
    r = instruct_radio(cmd);
  } while (r == RADIO_E_NONE && cn < num); 

  return r; 
}

/*
 * Decrease the volume.
 */
int
radio_quiet(int num)
{
  RadioCommand cmd[] = RADIO_DEC;
  int cn = 0, r = 0;

  do {
    r = instruct_radio(cmd);
  } while (r == RADIO_E_NONE && cn < num); 

  return r; 
}

/*
 * Change the frequency - see tune radio for the details!
 */
int
radio_set(float freq)
{
  if (!(legal_freq(freq))) {
    fprintf(stderr, "Oops: Frequency out of range (%.3fHz < x < %.3fHz)\n",
	    RADIO_MINHz, RADIO_MAXHz);
    return RADIO_E_RANGE;
  }
  else {
    return tune_radio(freq);
  }
}

/* This DOESN'T WORK!
 *  Maybe the timings are all wrong. It should be possible to do this - the
 *  dos program manages somehow. (We're after auto-tuning to the strongest
 *  signal
 */
int
radio_read(void)
{
  RadioCommand cmd1 = {0x00F8, 3};
  RadioCommand cmd2 = {0x00E8, 3};
  unsigned long int val = 0;

  if (allow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    fprintf(stderr, "You arn't being allowed access to the ports.\n");
    return RADIO_E_PERM;
  }

  write_word(cmd1.value, RADIO_BASEIO + cmd1.portoffset);
  /*usleep(RADIO_SLEEP);*/

  val = read_word(RADIO_BASEIO + 3);
  printf("Read 16bit port 0x%03X and got 0x%04lX\n", RADIO_BASEIO+3, val);
  val = read_word(RADIO_BASEIO + 3);
  printf("Read 16bit port 0x%03X and got 0x%04lX\n", RADIO_BASEIO+3, val);
  usleep(1000);
  val = read_word(RADIO_BASEIO + 3);
  printf("Read 16bit port 0x%03X and got 0x%04lX\n", RADIO_BASEIO+3, val);

  write_word(cmd2.value, RADIO_BASEIO + cmd2.portoffset);
  /*usleep(RADIO_SLEEP);*/
  
  if (disallow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    return RADIO_E_PERM;
  }
  return RADIO_E_NONE;
}
  


/*
 * Check for a legal frequency value.
 * Returns 1 for allowed, 0 for out of range.
 */
static int
legal_freq( float freq )
{
  if (freq < RADIO_MINHz || freq > RADIO_MAXHz)
    return 0;
  else
    return 1;
}

/*
 * Uses the `portlib.c' functions to open port access and write data to the
 * radio card. The sleep period seems to be essestial for most commands
 * except the frequency setting sequences - for these we have an alternative
 * function with no sleep period.
 */
static int
instruct_radio ( RadioCommand cmd[] )
{
  int cn = 0;

  if (allow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    fprintf(stderr, "You arn't being allowed access to the ports.\n");
    return RADIO_E_PERM;
  }

  while (cmd[cn].portoffset != 0xFF) {
    write_byte(cmd[cn].value, RADIO_BASEIO + cmd[cn].portoffset);
    usleep(RADIO_SLEEP);
    ++cn;
  }

  if (disallow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    return RADIO_E_PERM;
  }
  return RADIO_E_NONE;
}

static int
instruct_radio_fast ( RadioCommand cmd[] )
{
  int cn = 0;
  while (cmd[cn].portoffset != 0xFF) {
    write_word(cmd[cn].value, RADIO_BASEIO + cmd[cn].portoffset);
    ++cn;
  }
  return RADIO_E_NONE;
}

/*
 * The frequency value is split into bits and each bit encoded to four values
 * to be sent to the card.
 */
static int
tune_radio( float freq )
{
  RadioCommand BitSet[]  = RADIO_HZ1;
  RadioCommand BitClr[]  = RADIO_HZ0;
  RadioCommand FreqEnd[] = RADIO_HZE;
  unsigned long int bits;
  int cn;
  
  bits = (freq * 40) + 0x00A001AC;

  if (allow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    fprintf(stderr, "Oops. You don't have access to the ports.\n");
    return RADIO_E_PERM;
  }

  for (cn = 0; cn < 24; ++cn) {
    if (bits & 1)
      instruct_radio_fast(BitSet);
    else
      instruct_radio_fast(BitClr);
    bits>>=1;
  }

  if (disallow_io(RADIO_BASEIO, RADIO_NUMIO) == -1) {
    return RADIO_E_PERM;
  }

  return instruct_radio(FreqEnd);
}

